# Global login package...
#
# For scraping AT&T web pages protected by Global Login.
# 
# Usage:
# - enter a line in ~/.x3270rc of the form:
# glogin	hrid	pin
# where hrid and pin are your hrid and pin.  Please use tab as a separator.
# 
# - calling sequence
# use LWP::UserAgent;
# use HTTP::Cookies;
# use HTML::TableExtract;
# use HTML::Parser;
# 
# use lib qw(/home/hpk/lib/Perl/);
# use Glogin;
# 
# $ua = new LWP::UserAgent();
# $ua->cookie_jar(HTTP::Cookies->new());
# $ua->agent("Microsoft Internet Explorer/5.5");
# 
# glogin($ua);
# 
# $url = the url protected by global login
# $request = new HTTP::Request('GET' => $url);
# ...
# 

package Glogin;
 
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);
  
require Exporter;
   
@ISA = qw(Exporter AutoLoader);
@EXPORT = qw(glogin gcookie host_lookup);
$VERSION = '0.01';

use Getopt::Std;
use HTTP::Request::Common qw(POST);
use LWP::UserAgent;

sub glogin
{
	my ($browser, $tretURL) = @_;
	my $retURL;

	$up_pairs = host_lookup(undef, "glogin");
	foreach $tuple (@$up_pairs) {
		($userid, $password) = @$tuple;
	}

	$target_url='https://www.e-access.att.com/empsvcs/hrpinmgt/pagLogin/';

	$req = HTTP::Request->new(GET => $target_url);

	$webdoc = $browser->request($req);
	if(!$webdoc->is_success() ) {
	    $message = $webdoc->status_line;
	    # die "Failed $message\n";
	} else {
	    
	#	print STDERR "Page hit, these headers present on response -------\n";
	#	print STDERR $webdoc->headers_as_string();
	#	print STDERR "End headers ---------------------------------------\n";
	#	print $webdoc->content();
	#	die;
	}


	$url = 'https://www.e-access.att.com/pkmslogin.form';

	$req = POST $url , [
			    username => $userid,
			    password => $password,
			    btnSubmit => "  OK  "
			    ];

	$webdoc = $browser->request($req);


	if(!$webdoc->is_success() ) {
	    #die $webdoc->headers_as_string();
	    if ($webdoc->headers_as_string()=~/Location: (.+)/) {
		$url=$1;
		#print "=============================> $url\n";
		$req = POST $url , [];
		$webdoc = $browser->request($req);
		$message = $webdoc->status_line;
		#print STDERR $webdoc->headers_as_string();
		#print $webdoc->content();
		#die;
		if ($webdoc->content()=~/\"\.\..(.+?)\"/) {
		    $retURL="https://www.e-access.att.com/empsvcs/$1";
		    
		    #die $retURL;
		    $req = new HTTP::Request('GET', $retURL);
		    $webdoc = $browser->request($req);
		    # print $webdoc->content();
		    return;	# OK!
		} else { die "error ";}
	    } else {die 'error';}
	}; # else {die 'error';}
}

sub gcache {
	my ($ua, $jar, $ret);

	$age = -M ".glogin";
	if (defined $age and $age <= 0.9)
	{
		open GLOGIN, ".glogin";
		$ret = "# from cache\n";
		while (<GLOGIN>)
		{
			$ret = $ret . $_;
		}
		return $ret;
	}
	# sysopen GLOGIN, ".glogin", O_WRONLY|O_CREAT|O_TRUNC, 0600 or die "Error: $ERRNO";
	open GLOGIN, "> .glogin";
	$ua = new LWP::UserAgent();
	$jar = HTTP::Cookies->new();
	$ua->cookie_jar($jar);
	$ua->agent("Microsoft Internet Explorer/5.5");
 
	glogin($ua);

        while (($key,$array) = each %{$jar->{'COOKIES'}{'.att.com'}}) {
		foreach $var (keys %$array)
		{
			$ret = $ret . "newcookie('$var', '", $array->{$var}[1], "');\n";
			print GLOGIN "newcookie('$var', '", $array->{$var}[1], "');\n";
		}
	}
	return $ret;
}

sub host_lookup {
    my $self = shift;
    my $host = shift;
    my ($username, $password) = @_;
    my ($d, $file, $line, $h, $u, $p, @up_pairs);
    my $base = ".x3270rc";
    my @dirs = qw( . );

    push (@dirs, $ENV{HOME}) if (defined($ENV{HOME}));
    foreach $d (@dirs) {
	$file = "$d/$base";
	if (open(FH, "< $file")) {
	    while ($line = <FH>) {
		next if ($line =~ /^\s*#/);
		chomp($line);
		next unless (($h, $u, $p) = split(" ", $line, 3));
		next unless (($h eq $host) && defined($p));
		next if (defined($username) && ($username ne $u));
		# host matches, and either username matches,
		# or no username was passed
		push(@up_pairs, [ $u , $p ]);
	    }
	}
    }
    print STDERR "ERROR: No $host entry in your ~/.x3270rc file\n" unless @up_pairs;
    return \@up_pairs;
}

