import java.io.IOException;
import net.jxta.j2me.Element;
import net.jxta.j2me.Message;
import net.jxta.j2me.PeerNetwork;
import java.util.Hashtable;
import java.util.Date;

//*********************************************************
//*  SimplePresence Service for JXTA - Designed for TINI
//*
//*  (c) 2002, All Rights Reserved,  SL DDJ
//*  Sample Code to accompany Dr. Dobbs Journal Article

public class DDJFinder {

     private static final String serviceName = "ddj.simple.presence";
     private static final String IP2P_PIPE_ID = "urn:jxta:uuid-"
		+ "59616261646162614E50472050325033D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D104";
     private static final String IP2P_GROUP_NAME = "IP2PGRP";
     private static final String IP2P_PIPE_NAME = "JxtaTalkUserName." + IP2P_GROUP_NAME;

     private String proxyURL = "http://localhost:9700";  // default local
     private Hashtable presenceDB = new Hashtable(200);
     private PeerNetwork peerNetwork = null;

    public DDJFinder(String proxy) {
        proxyURL = proxy;
     
          }

  public static void main(String[] argv) {

		if (argv.length != 1) {
			System.err.println("Usage: DDJFinder proxyServerURL");
			System.exit(1);
		}
           DDJFinder myFinder = new DDJFinder(argv[0]);
           myFinder.init();
           myFinder.processMessages();

	}

    public void init() {
	try {
			System.err.println("Creating peer network...");
			peerNetwork = PeerNetwork.createInstance(serviceName);
			System.err.println("Connecting to " + proxyURL);
  			peerNetwork.connect(proxyURL, null);
            }
	   catch (Exception ex) {
				System.err.println("Problem connecting to JXTA network...");			            	ex.printStackTrace();
			}
          	
        }
	
   public void processMessages()  {

  		try {


                  System.err.println("Listening to pipe " + IP2P_PIPE_ID);
			peerNetwork.listen(serviceName, IP2P_PIPE_ID, "JxtaPropagate");
			System.err.println("Ready.");

			while(true) {
				Message msg = pollForMsg();
                        if (! isCommand(msg))
                               logSender(msg);
                        else
                            {
                                if (!isTerminate(msg))
                             sendMessage(peerNetwork, processCmd(msg), serviceName   );                         
                                 else
                                     break;
                             }
                       }// of while
				



			
  		} catch (Exception ex) {
			System.err.println("Error during message processing");
			ex.printStackTrace();
			System.exit(1);
		}


       }


     public Message pollForMsg() {
          Message retMsg = null;
           try {
                while (true) {
                    retMsg = peerNetwork.poll(0);
                    if (retMsg != null)
                         break;
                 }
         } catch (Exception ex) {
            System.err.println("Error while polling for message.");
            ex.printStackTrace();
            System.exit(1);
            }
            return retMsg;
    }


    public boolean isCommand(Message msg) {
              for (int i = 0; i < msg.getElementCount(); ++i) {
					Element e = msg.getElement(i);
                              if (e.getName().equals("JxtaTalkSenderMessage")) {
					String msgtxt = new String(e.getData());
                              if (msgtxt.endsWith("?") || msgtxt.endsWith("+")  
                                          || msgtxt.endsWith("-") ||
                                            msgtxt.endsWith("xxx"))
                                  return(true);
                              else
                                  return(false);
                              }
                         }
           return(false);
           }
    public boolean isTerminate(Message msg) {
              for (int i = 0; i < msg.getElementCount(); ++i) {
					Element e = msg.getElement(i);
                              if (e.getName().equals("JxtaTalkSenderMessage")) {
					String msgtxt = new String(e.getData());
                              if (msgtxt.endsWith("xxx"))
                                  return(true);
                              else
                                  return(false);
                               } //if
                       } // for
                     return(false);             
     }
				
      public String processCmd(Message msg) {
          String retStr = "";
         for (int i = 0; i < msg.getElementCount(); ++i) {
					Element e = msg.getElement(i);
                              if (e.getName().equals("JxtaTalkSenderMessage")) {
					String msgtxt = new String(e.getData());
                              String nameToMonitor = msgtxt.substring(0, msgtxt.length() - 1); 
                              if (msgtxt.endsWith("?")) {
                                   // query service
                                  if (presenceDB.containsKey(nameToMonitor))
                                      retStr = nameToMonitor + " was last seen at " + 
                                           ((Date) presenceDB.get(nameToMonitor)).toString();
                                   else
                                      retStr = "Sorry " + nameToMonitor + " is not tracked.";   
                                 break; 
                                   }
                               if (msgtxt.endsWith("+")) {
                                  // add to service
                                 if (presenceDB.containsKey(nameToMonitor))
                                   retStr = nameToMonitor + " is already being tracked";
                                  else  {
                                       presenceDB.put(nameToMonitor, " never ");
                                   retStr = nameToMonitor + " is now tracked";

                                        }
                                     break;
                                    }
					if (msgtxt.endsWith("-")) {
                                  // remove from service
                                 if (presenceDB.containsKey(nameToMonitor)) {
                                       presenceDB.remove((Object) nameToMonitor);
                                        System.gc();
                                   retStr = nameToMonitor + " is no longer tracked";
                                      }
                                  else
                                     retStr = nameToMonitor + " is not currently tracked";
                                 break;  
                                }
                            } // if
                 }// for

           return retStr;
        }

            public void logSender(Message msg) {
             for (int i = 0; i < msg.getElementCount(); ++i) {
					Element e = msg.getElement(i);
		           if (e.getName().equals("JxtaTalkSenderName")) {
						String sender = new String(e.getData());
                              if (presenceDB.containsKey((Object) sender))
                                    presenceDB.put(sender, new Date());
                           break;
                           }
			
                  }
              }
	

  	
	private static void sendMessage(PeerNetwork peerNetwork, String val, String usr) throws IOException {
		Element[] elements = new Element[3];
		elements[0] = new Element("JxtaTalkSenderName", usr.getBytes(),null, null);
		elements[1] = new Element("JxtaTalkSenderMessage", val.getBytes(),null, null);
		elements[2] = new Element("GrpName", "NetPeerGroup".getBytes(), null,null);
		Message msg = new Message(elements);
		peerNetwork.send(IP2P_PIPE_NAME, IP2P_PIPE_ID, "JxtaPropagate", msg);
	}
	

}
