/*
 *  Lightweight Embedded XML-RPC Server main
 *
 *  mtj@cogitollc.com
 *
 */

#include <sys/socket.h>
#include <sys/time.h>
#include <arpa/inet.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/un.h>
#include <unistd.h>
#include "xmlparser.h"

const char *notimplemented={"HTTP/1.1 501 Not Implemented\n\n"};
const char *success={"HTTP/1.1 200 OK\n"};
const char *separator={"\015\012\015\012"};

#define MAX_ELEMENT	64


/*---------------------------------------------------------------------------
 * findMsgBody() - Find the message body of an HTTP Request Message
 *-------------------------------------------------------------------------*/
char *findMsgBody( char *buf )
{
  char *temp;

  temp = strstr(buf, separator);

  if (temp == NULL) return NULL;
  else return temp+4;
}


/*---------------------------------------------------------------------------
 * getHeaderElement() - Find the HTTP header and return it's value.
 *-------------------------------------------------------------------------*/
int getHeaderElement( char *buffer, char *header, char *value, int size )
{
  char *temp;
  int  i=0;

  temp = strstr(buffer, header );
  if (temp) {

    /* Skip the header element */
    temp += strlen(header);

    /* Skip any white-space */
    while (*temp == ' ') temp++;

    /* Copy the rest to the value parameter */
    while ((*temp != ' ') && (*temp != '\n') && (i < size)) {
      value[i++] = *temp++;
    }
    value[i] = 0;

    return i;

  }

  return -1;
}


/*---------------------------------------------------------------------------
 * handleConnection() - Parse and handle the current HTTP request message
 *-------------------------------------------------------------------------*/
void handleConnection(int fd)
{
  fd_set rfds;
  struct timeval tv;
  int ret, len, max=0, payloadLength = -1;
  char buffer[4096]={0}, value[MAX_ELEMENT+1];
  char *temp;

  /*
   * Read in the Request Header 
   */
  do {

    FD_ZERO(&rfds);
    FD_SET(fd, &rfds);

    tv.tv_sec = 1;
    tv.tv_usec = 0;

    ret = select(fd+1, &rfds, NULL, NULL, &tv);

    if (ret > 0) {

      if (FD_ISSET(fd, &rfds)) {

        len = recv( fd, &buffer[max], 1024, 0);

        if (len > 0) {
          max += len;
          buffer[max] = 0;

          ret = getHeaderElement(buffer, "Content-Length:", value, MAX_ELEMENT);
          if (ret > 0) payloadLength = atoi(value);

        } else {
          ret = -1;
          break;
        }

      }

    } else {
      /* Timeout... */
      ret = -1;
      break;
    }

    temp = strstr(buffer, separator);

    if (temp) {
      if (strlen(temp)-4 == payloadLength) break;
    }

  } while (1);

  /*
   * Determine request 
   */
  if (!strncmp(buffer, "POST", 4)) {
    temp = findMsgBody(buffer);
    parseMessage(fd, temp);
  } else {
    write(fd, notimplemented, strlen(notimplemented));
  }
}


/*---------------------------------------------------------------------------
 * main() - The embedded HTTP server main
 *-------------------------------------------------------------------------*/
int main(int argc, char *argv[])
{
  int listenfd, connfd, on=1;
  socklen_t clilen;
  struct sockaddr_in cliaddr, servaddr;

  extern void userInit();

  /* Init the dynamic content test func */
  userInit();

  listenfd = socket(AF_INET, SOCK_STREAM, 0);

  setsockopt( listenfd, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on) );

  bzero((void *)&servaddr, sizeof(servaddr));
  servaddr.sin_family = AF_INET;
  servaddr.sin_addr.s_addr = htonl(INADDR_ANY);
  servaddr.sin_port = htons(80);

  bind(listenfd, (struct sockaddr *)&servaddr, sizeof(servaddr));

  listen(listenfd, 5);

  for ( ; ; ) {

    clilen = sizeof(cliaddr);
    connfd = accept(listenfd, (struct sockaddr *)&cliaddr, &clilen);
    if (connfd <= 0) break;

    handleConnection(connfd);
    close(connfd);

  }

  close(listenfd);
  return(0);
}


/*
 *  Copyright (c) 2002 Cogito LLC.  All rights reserved.
 * 
 *  Redistribution and use in source and binary forms, with or 
 *  without modification, is hereby granted without fee provided 
 *  that the following conditions are met:
 * 
 *    1.  Redistributions of source code must retain the above 
 *        copyright notice, this list of conditions and the 
 *        following disclaimer.
 *    2.  Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the 
 *        following disclaimer in the documentation and/or other 
 *        materials provided with the distribution.
 *    3.  Neither the name of Cogito LLC nor the names of its 
 *        contributors may be used to endorse or promote products 
 *        derived from this software without specific prior 
 *        written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY COGITO LLC AND CONTRIBUTERS 'AS IS' 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED 
 * TO, THE IMPLIED WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL COGITO LLC 
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARAY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF 
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE.
 */

