/*
 *  Lightweight Embedded XML-RPC Server Types and Definitions
 *
 *  mtj@cogitollc.com
 *
 */

#ifndef XMLPRIV_H
#define XMLPRIV_H

typedef struct {
  char *buf;
  int  len;
  int  index;
} parseBuf_t;

#define TAG	0
#define VALUE	1
#define DONE	2

#define MAX_STRING	80

typedef struct {
  union {
    int i;
    char boolean;
    double d;
    char string[MAX_STRING+1];
  } u;
} arg_t;

#define MAX_ARGS	10
#define MAX_RESPONSE	2048

typedef struct {
  char  methodName[MAX_STRING];
  arg_t arguments[MAX_ARGS];
  char  argList[MAX_ARGS];
  int   argCount;
  int   argIndex;
  char  response[MAX_RESPONSE];
  int   error;
} xmlCall_t;

#define MAX_METHODS	10

typedef struct {
  int (*func)( xmlCall_t * );
  char methodName[MAX_STRING+1];
} methodEntry_t;

int parseMessage ( int sock, char *buffer );

int getIntegerArg( xmlCall_t *xmlCall, int *arg );
int getBooleanArg( xmlCall_t *xmlCall, int *arg );
int getDoubleArg( xmlCall_t *xmlCall, double *arg );
int getStringArg( xmlCall_t *xmlCall, char *arg );

int callUserFunc( xmlCall_t *xmlCall );

int xmlBuildResponse( xmlCall_t *, char *, ... );

#endif

/*
 *  Copyright (c) 2002 Cogito LLC.  All rights reserved.
 * 
 *  Redistribution and use in source and binary forms, with or 
 *  without modification, is hereby granted without fee provided 
 *  that the following conditions are met:
 * 
 *    1.  Redistributions of source code must retain the above 
 *        copyright notice, this list of conditions and the 
 *        following disclaimer.
 *    2.  Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the 
 *        following disclaimer in the documentation and/or other 
 *        materials provided with the distribution.
 *    3.  Neither the name of Cogito LLC nor the names of its 
 *        contributors may be used to endorse or promote products 
 *        derived from this software without specific prior 
 *        written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY COGITO LLC AND CONTRIBUTERS 'AS IS' 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED 
 * TO, THE IMPLIED WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL COGITO LLC 
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARAY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF 
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE.
 */

