#!/usr/common/bin/perl
#-------------------------------------------------------------------------
#--     wsp.pl    -    Web Scraping Proxy server.                       --
#--                                                                     --
#--     Howard P. Katseff, AT&T Laboratories                            --
#--     based on code by Dan Boneh, Stanford University                 --
#--	  (http://www-scf.usc.edu/~csci571/Special/HTTP/proxy.html)     --
#--                                                                     --
#--     To run, type                                                    --
#--         wsp.pl [-a] [-v] [port]                                     --
#--     at the shell prompt.                                            --
#--                                                                     --
#--	-a causes wsp.pl to save all web pages                          --
#--	-v verbose, for debugging wsp.pl                                --
#--                                                                     --
#-------------------------------------------------------------------------

use Socket;
use CGI qw(unescape);
use Getopt::Std;

require HTML::TokeParser;
use HTML::Form;

$ret = getopts('acvi');

$ignore_url = "\.jpg\$|\.gif\$|\.css\$" unless $opt_i;

#---  Define a friendly exit handler
$SIG{'KILL'} = $SIG{QUIT} = $SIG{INT} = 'exit_handler';
sub exit_handler {
    print "\n\n --- Proxy server is dying ...\n\n" if $opt_v;
    close(SOCKET);
    exit;
}
$SIG{'TTIN'} = 'IGNORE';
$SIG{'PIPE'} = 'IGNORE';

$| = 1;
$is_http = 1;
$is_https = 0;

if ($0 =~ /(^.*)\//)
{
	$progdir = $1;
}
else
{
	$progdir = '.';
}

unless (-e "$progdir/$server-key.pem")
{
	if ($link = readlink $0 and $link =~ /(^.*)\//)
	{
		$link = $1;
		if ($link =~ /^\//)
		{
			$progdir = $link;
		} else {
			$progdir .= $link;
		}
	}
}

unless (-e "$progdir/server-key.pem")
{
	print "Can't find server-key.pem.  HTTPS will not be available\n";
	$no_https = 1;
}

use Net::SSLeay qw(die_now die_if_ssl_error);
Net::SSLeay::load_error_strings();
Net::SSLeay::SSLeay_add_ssl_algorithms();
Net::SSLeay::randomize("/etc/motd");

$ofilecnt = 0;
while (-e "w$ofilecnt")
{
	$ofilecnt++;
}

$proxy_port = shift(@ARGV);
$proxy_port = 5364 unless $proxy_port =~ /\d+/;

#---  Setup socket
$socket_format = 'S n a4 x8';
&listen_to_port(SOCKET, $proxy_port);
$local_host = `hostname`;
chomp $local_host;
$local_host_ip = (gethostbyname($local_host))[4];
print " --- Proxy server running on $local_host port: $proxy_port \n\n";

unless ($no_https)
{
	$ctx = Net::SSLeay::CTX_new ()		 or die_now("CTX_new ($ctx): $!");
	Net::SSLeay::CTX_set_options($ctx, &Net::SSLeay::OP_ALL)
		 and die_if_ssl_error("ssl ctx set options");

	# Following will ask password unless private key is not encrypted
	Net::SSLeay::CTX_use_RSAPrivateKey_file ($ctx, "$progdir/server-key.pem", &Net::SSLeay::FILETYPE_PEM);
	die_if_ssl_error("private key");
	Net::SSLeay::CTX_use_certificate_file ($ctx, "$progdir/server-cert.pem", &Net::SSLeay::FILETYPE_PEM);
	die_if_ssl_error("certificate");
}

vec($rin, fileno(SOCKET), 1) = 1;
vec($rin, 0, 1) = 1;


#---  Loop forever taking requests as they come
while (1) {
#    $nfound = select ($rout=$rin, undef, undef, undef);
#    if (vec($rout, 0, 1))
#    {
#	# process annotation
#	$in = <STDIN>;
#	print "## $in\n" if $in;
#	next;
#    }
#    elsif (!vec($rout, fileno(SOCKET), 1))
#    {
#	print "### ??? nothing selected in $rout, $nfound\n";
#	next;
#    }

    print " --- Accepting connection ...\n" if $opt_v;
    ($addr = accept(CLIENT,SOCKET)) || die "accept $!";
    ($port,$inetaddr) = (unpack($socket_format,$addr))[1,2];
    @inetaddr = unpack('C4',$inetaddr);
    print "Connection from ", join(".", @inetaddr), "  port: $port \n" if $opt_v;
#---  Read first line of request and analyze it.
#---  Return an edited version of the first line and the request method.
   ($first,$method) = &analyze_request;
#---  Send request to remote host

    next unless $first;

    writeserver($first);
    print "FIRST LINE: $first" if $opt_v;

    $content_length = 0;
    while ($_ = readclient()) {
        print "INPUT: $_" if $opt_v;
	if ( /^keep-alive/i || /proxy-connection: *keep-alive/i)
	{
		print "Ignoring keep-alive\n" if $opt_v;
		next;
	}
	if ( /^Referr*er:/ and not $url =~ /$ignore_url/)
	{

		$line = $_;
		$line =~ s/
$//;
		print "# $line";
	}
	if ( /^Cookie:/ and not $url =~ /$ignore_url/)
	{
		$xcookie = $_;
		$xcookie =~ s/^Cookie: *//;
		chomp $xcookie;
		$xcookie =~ s/
$//;
		foreach $cookie (split "; ", $xcookie)
		{
			$cookie =~ /^([^=]*)=(.*)$/;
			$var = $1;
			next if $var eq 'attESSec' or $var eq 'attESHr';
			$val = $2;
			$val = unescape($val);
			if ($cookie{$var} eq $val)
			{
				print "# Cookie: '$var', '$val'\n";
			}
			else
			{
				unless ($saw_cookie{$var})
				{

					foreach $line (split '\n', $content)
					{
						print "# PREVIOUS JS: $line\n" if $line =~ /$var/;
					}
					$saw_cookie{$var} = 1;
					if ($javascript_text)
					{
						print "# Saving previous web page as w$ofilecnt\n";
						while (-e "w$ofilecnt")
						{
							$ofilecnt++;
						}
						open OFILE, "> w$ofilecnt" or warn;
						print OFILE "$prev_url\n";
						print OFILE $prev_content;
						close OFILE;
						$ofilecnt++;
						$javascript_text = "";
					}
					print "# Cookie (NO Set-Cookie): '$var', '$val'\n";
				} else {
					print "# Cookie: '$var', '$val'\n";
				}
			}
		}
	}
	if (/Content-type:.*boundary=/)
	{
		$boundary = $_;
		chomp $boundary;
		$boundary =~ s/.*boundary=//;
		$boundary =~ s/
$//;
	}

        next if (/Proxy-Connection:/);

	if (/Content-length: *(\d*)/i) {
	    $content_length = $1;
	}

        writeserver("Connection: close\n") if ($is_chunked and $_ =~ /^[\s\x00]*$/);
        writeserver( $_);
        last if ($_ =~ /^[\s\x00]*$/);
    }
    print "END OF HEADER ON READCLIENT: METHOD is $method, LENGTH is $content_length\n" if $opt_v;
    if ($method eq "POST") {
	if ($content_length != 0) {
	    read CLIENT, $data, $content_length if $is_http;
	    $data = readclient($content_length) if $is_https;
	} else {
	    $data = readclient();
	}
        print "POST body: $data\n" if $opt_v;
        writeserver( $data);

	if ($data =~ /[[:cntrl][:^ascii:]]/ or !($data =~ /=/))
	{
		print "# Non-standard POST request.  Saving body in w$ofilecnt\n";
		while (-e "w$ofilecnt")
		{
			$ofilecnt++;
		}
		open OFILE, "> w$ofilecnt" or warn;
		print OFILE "$data";
		close OFILE;
		$ofilecnt++;
		$save_response = 1;
	}
	else
	{
		print "\$request = POST \"$url\" , [\n";
		@data = split "\n", $data;
		undef $once;
		undef %current_form;
		for (;;) {
			$line = shift @data;
			$line =~ s/
$//;
			last unless defined $line;
			if ($line =~ s/Content-Disposition:.*form-data;.*[\W]name=//)
			{
				$var = $line;
				$var =~ s/;.*//;
				$value = "";
				for (;;)
				{
					$line = shift @data;
					$line =~ s/
$//;
					last unless defined $line;
					last if $line =~ /$boundary/;
					$value = "$value$line";
				}
				$var =~ s/"/'/g;
				$value = unescape($value);
				print "\t$var => \"$value\",\n";
				$current_form{$var} = $value;
			}
			elsif (! $once) {
				$once = 1;
				next if $boundary && $line =~ /$boundary/;
				@param = split '\&', $line;
				foreach $param (@param)
				{
					($var, $value) = split '=', $param;
					$value = unescape($value);
					print "\t\'$var\' => \"$value\",\n";
					$current_form{$var} = $value;
				}
			}

		}
		print "] ;\n";

		# compare with previous form response
		undef $best_form;
		$best_count = -1;
		$form_number = 0;
		if (scalar (keys %current_form) > 0)
		{
			foreach $form (@forms)
			{
				$count = 0;
				for $input ($form->inputs)
				{
					$count++ if defined $current_form{$input->name};
				}
				if ($count > $best_count)
				{
					$best_count = $count;
					$best_form = $form;
					$best_number = $form_number;
				}
				$form_number++;
			}
			if (defined $best_form)
			{
				for $input ($best_form->inputs)
				{
					my @fnv = $input->form_name_value();
					$best_form{$input->name} = $fnv[1] if scalar(@fnv) == 2;
				}
				print "# DIFFERENCES between form $best_number from server and submitted form:\n";
				print "\$post_args = { };\n";
				for $element (keys %current_form)
				{
					if ($best_form{$element} ne $current_form{$element})
					{
						print "\$post_args->{'$element'} = \"$current_form{$element}\";  # was \"$best_form{$element}\"\n";
					}
					else
					{
						# print "## \$post_args->{'$element'} unchanged.  Was \"$best_form{$element}\"\n";
					}
				}
				print "# end DIFFERENCES\n";
			}
		}
		else
		{
			print "# form had no elements\n";
		}
	}
    }
    elsif ($method eq "GET" || $method eq "HEAD") {
	print "\$request = new HTTP::Request('$method' => \"$url\");\n" unless $url =~ /$ignore_url/ ; 
    }
    else
    {
	print "Request with unknown method $method\n";
    }
    undef $boundary;
    # writeserver("\n");

#---  Wait for response and transfer it to requestor.
    print " --- Done sending. Response: \n\n" if $opt_v;
    undef $no_content;
    $next_to_last = 0;
    $is_chunked = 0;
    $header = 1;
    $text = 0;
    $prev_url = $url unless $url =~ /$ignore_url/;
    ;
    $content = "";
    $content_length = 0;
    readserver_undef_remain();
    while ($_ = readserver()) {
        writeclient($_);
        if ($header || $text) {   
            # print $_ if $opt_v;
            if ($status != 100 && $header && $_ =~ /^[\s\x00]*$/) {
		print "END HEADER\n" if $opt_v;
                $header = 0;
		readserver_set_remain($content_length) if ($content_length || $no_content);
           }
           if ($header && $_ =~ /^Content-type: text/i) {
               $text = 1;
           }
        }
	s/
$//;

	print "RESPONSE: $_" if $header && $opt_v;
	if (/^HTTP\/[^ ]* (\d*)/)
	{
		$status = $1;
		$no_content = 1 if ($status == 204) or ($status == 304);
		print "STATUS -$status-$no_content\n" if $opt_v;
	}
	if (/Transfer-Encoding: *chunked/i)
	{
		$is_chunked = 1;
	}

	unless ($url =~ /$ignore_url/)
	{
		if ($header)
		{
			print "# $_" if /^HTTP/ && ! /200/;
			if (/^Set-Cookie:/i)
			{
				print "# $_";
				$setline = $_;
				$setline =~ s/Set-Cookie: *//i;
				$setline =~ /^([^=]*)=([^;]*);/;
				$var = $1;
				$val = $2;
				$cookie{$var} = unescape($val);
			}
			$content = "$content$_";
		}
		elsif ($text)
		{
			$content = "$content$_" unless $url =~ /$ignore_url/;
		}
	}
	if (s/^Content-length: *//i)
	{
		print "# Content-length: $_" if $opt_v;
		$content_length =$_;
	}
    }
    if ($content)
    {
	$tableno = 0;
	$rowcnt = 0;
	$nesting = 0;
	$maxnest = 0;
	$javascript = 0;
	#print "CONTENT: $content\n";
	$p = HTML::TokeParser->new( \$content );
	while (my $token = $p->get_token)
	{
		$tok = $token->[0] . $token->[1];
		if ($tok =~ /^SSCRIPT/i &&  $token->[2]{language} =~ /JAVASCRIPT/i)
		{
			$javascript_text = $p->get_text. "\n";
			$javascript = 1;
		}
		if ($tok =~ /^STABLE/i)
		{
			$tableno++ if $nesting == 0;
			$nesting++;
			$maxnest = $nesting if $nesting > $maxnest;
		}
		if ($tok =~ /^ETABLE/i)
		{
			$nesting--;
			if ($nesting == 0)
			{
				print "# Table $tableno: $rowcnt rows";
				print "; table nesting: $maxnest" if $maxnest > 1;
				print "\n";
				$rowcnt = 0;
				$maxnest = 0;
			}
		}
		if ($tok =~ /^STR/i)
		{
			$rowcnt++;
		}
	    }
	    print "# Contains JavaScript\n" if $javascript;
	    if ($opt_a || $tableno || $rowcnt || $save_response)
	    {
		if ($save_response)
		{
			print "# Saving response as w$ofilecnt\n";
		}
		else
		{
			print "# Saving web page as w$ofilecnt\n";
		}
		$save_response = 0;
		while (-e "w$ofilecnt")
		{
			$ofilecnt++;
		}
		open OFILE, "> w$ofilecnt" or warn;
		print OFILE "$url\n";
		print OFILE $content;
		close OFILE;
		$ofilecnt++;
	    }
    }

    @ff = HTML::Form->parse($content, $url);
    @forms = @ff if @ff;

    if ($is_http)
    {
	    close(SERVER);
	    close(CLIENT);
    }
    if ($is_https)
    {
	print "CLOSING\n" if $opt_v;
	CORE::shutdown SERVER, 1;
	Net::SSLeay::free ($sslserver);
	Net::SSLeay::CTX_free ($ctxserver);
	close SERVER;

	CORE::shutdown CLIENT, 1;
	Net::SSLeay::free ($sslclient);
	close CLIENT;

    }

    print "\n"  unless $url =~ /$ignore_url/;
    $prev_content = $content unless $url =~ /$ignore_url/;
}
#-------------------------------------------------------------------------
#--     analyze_request                                                 --
#--                                                                     --
#--     Analyze a new request.  First read in first line of request.    --
#--     Read URL from it, process URL and open connection.              --
#--     Return an edited version of the first line and the request      --
#--     method.                                                         --
#-------------------------------------------------------------------------
sub analyze_request {
#---  Read first line of HTTP request
    $first = <CLIENT>;
    $first_save = $first;

    if ($first =~ /CONNECT/)
    {
	if ($no_https)
	{
		print "# Can't process HTTPS request\n\n";
		return "";
	}
	else
	{
		return analyze_https_request($first);
	}
    }

    $url = ($first =~ m|(http://\S+)|)[0];
    print "Request for URL:  $url \n" if $opt_v;
    print "# Request: $url \n" unless $url =~ /$ignore_url/;

#---  Check if first line is of the form GET http://host-name ...
    ($method, $remote_host, $remote_port) = 
        ($first =~ m!(GET|POST|HEAD) http://([^/:]+):?(\d*)! );
#---  If not, bad request.
    
    if (!$remote_host) {
        print $first if $opt_v;
        while ($_ = <CLIENT>) {
            print $_ if $opt_v;
            last if ($_ =~ /^[\s\x00]*$/);
        }
        print "# Ignoring invalid HTTP request from ", join(".", @inetaddr), "\n\n";
#       print CLIENT "Content-type: text/plain","\n\n";
        print CLIENT "I don't understand your request.\n";
        close(CLIENT);
	return;
    }
#---  If requested URL is the proxy server then ignore request
    $remote_ip = (gethostbyname($remote_host))[4];
    if (($remote_ip eq $local_host_ip) && ($remote_port eq $proxy_port)) {
        print $first;
        while ($_ = <CLIENT>) {
            print $_;
            last if ($_ =~ /^[\s\x00]*$/);
        }
        print " --- Connection to proxy server ignored.\n";
#       print CLIENT "Content-type: text/plain","\n\n";
        print CLIENT "It's not nice to make me loop on myself!.\n";
        close(CLIENT);
        exit;
    }
#---  Setup connection to target host and send request
    $remote_port = "http" unless ($remote_port);
    &open_connection(SERVER, $remote_host, $remote_port);
#---  Remove remote hostname from URL
        $first =~ s/http:\/\/[^\/]+//;
    $is_http = 1;
    $is_https = 0;
    ($first, $method);
}

sub analyze_https_request {

	my $input = $_[0];
	print "HTTPS: $input\n" if $opt_v;
	while (<CLIENT>)
	{
		chomp;
		s/
//;
		last unless $_;
		print "Ignore: $_\n" if $opt_v;
	}

	if ($input =~ /CONNECT ([^:]*):([^ ]*) /)
	{
		$remote_host = $1;
		$remote_port = $2;
		print "DEST: $remote_host -- $remote_port\n" if $opt_v;
	        $cur = select (CLIENT); $| = 1; select ($cur);

		print CLIENT "HTTP/1.0 200 Connection established\nProxy-agent: Netscape-Proxy/1.1\n\n";
	}
	else
	{
		print "ERROR: Invalid CONNECT request: $input\n";
		while (<CLIENT>)
		{
			chomp;
			print "INPUT: $_\n" if $opt_v;
		}
		return;
	}

	# We now have a network connection, lets fire up SSLeay...

	$sslclient = Net::SSLeay::new($ctx)	  or die_now("SSL_new ($sslclient): $!");
	Net::SSLeay::set_fd($sslclient, fileno(CLIENT));

	Net::SSLeay::accept($sslclient) and die_if_ssl_error('ssl accept');
	die_if_ssl_error("ssl client accept");
	print "Cipher `" . Net::SSLeay::get_cipher($sslclient) . "'\n" if $opt_v;
        print "Client certificate: ", Net::SSLeay::dump_peer_certificate($sslclient) if $opt_c;

	$is_http = 0;
	$is_https = 1;

#---  Read first line of HTTP request
	$first = readclient();
	print "FIRST INPUT LINE: $first\n" if $opt_v;
	($method, $url) = 
		($first =~ m!(GET|POST|HEAD) ([^ ]*) ! );
	$url = "https://$remote_host$url";
	print "Request for URL: $url \n" if $opt_v;
	print "# Request: $url \n" unless $url =~ /$ignore_url/;
#---  If not, bad request.
	    
	if (!$remote_host) {
		print $first if $opt_v;
		while (readclient()) {
		    print $_ if $opt_v;
		    last if ($_ =~ /^[\s\x00]*$/);
		}
		print "Invalid HTTP request from ", join(".", @inetaddr), "\n";
	#       print CLIENT "Content-type: text/plain","\n\n";
		print CLIENT "I don't understand your request.\n";
		close(CLIENT);
		exit;
	}
#---  If requested URL is the proxy server then ignore request
	$remote_ip = (gethostbyname($remote_host))[4];
	if (($remote_ip eq $local_host_ip) && ($remote_port eq $proxy_port)) {
		print $first;
		while (readclient()) {
		    print $_;
		    last if ($_ =~ /^[\s\x00]*$/);
		}
		print " --- Connection to proxy server ignored.\n";
	#       print CLIENT "Content-type: text/plain","\n\n";
		print CLIENT "It's not nice to make me loop on myself!.\n";
		close(CLIENT);
		exit;
	    }
#---  Setup connection to target host and send request
	$remote_port = "https" unless ($remote_port);
	&open_https_connection(SERVER, $remote_host, $remote_port);
#---  Remove remote hostname from URL
	$first =~ s/https:\/\/[^\/]+//;

	($first, $method);
}

#-------------------------------------------------------------------------
#--     listen_to_port(SOCKET, $port)                                   --
#--                                                                     --
#--     Create a socket that listens to a specific port                 --
#-------------------------------------------------------------------------
sub listen_to_port {
    local ($port) = $_[1];
    local ($socket_format, $proto, $packed_port, $cur, $max_requests);
    $max_requests = 3;          # Max number of outstanding requests
    $socket_format = 'S n a4 x8';
    $proto = (getprotobyname('tcp'))[2];
    $packed_port = pack($socket_format, &AF_INET, $port, "\0\0\0\0");
    socket($_[0], &PF_INET, &SOCK_STREAM, $proto) || die "socket: $!";
    bind($_[0], $packed_port) || die "bind: $!";
    listen($_[0], $max_requests) || die "listen: $!";
    $cur = select($_[0]);  
    $| = 1;                             # Disable buffering on socket.
    select($cur);
    }

#-------------------------------------------------------------------------
#--     open_connection(SOCKET, $remote_hostname, $port)                --
#--                                                                     --
#--     Create a socket that connects to a certain host                 --
#--     $local_host_ip is assumed to be local hostname IP address       --
#-------------------------------------------------------------------------
sub open_connection {
    local ($remote_hostname, $port) = @_[1,2];
    local ($socket_format, $proto, $packed_port, $cur);
    local ($remote_addr, @remote_ip, $remote_ip);
    local ($local_port, $remote_port);
    if ($port !~ /^\d+$/) {
        $port = (getservbyname($port, "tcp"))[2];
        $port = 80 unless ($port);
    }
    $proto = (getprotobyname('tcp'))[2];
    $remote_addr = (gethostbyname($remote_hostname))[4];
    if (!$remote_addr) {
        warn "Unknown host: $remote_hostname";
	return;
    }

    @remote_ip = unpack("C4", $remote_addr);
    $remote_ip = join(".", @remote_ip);
    print "Connecting to $remote_ip port $port.\n\n" if $opt_v;
    $socket_format = 'S n a4 x8';
    $local_port  = pack($socket_format, &AF_INET, 0, $local_host_ip);
    $remote_port = pack($socket_format, &AF_INET, $port, $remote_addr);
    socket($_[0], &AF_INET, &SOCK_STREAM, $proto) || die "socket: $!";
    bind($_[0], $local_port) || die "bind: $!";
    connect($_[0], $remote_port) || die "socket to $remote_ip: $!";
    $cur = select($_[0]);  

    $| = 1;                             # Disable buffering on socket.
    select($cur);
}

sub open_https_connection {
    local ($remote_hostname, $port) = @_[1,2];
    local ($socket_format, $proto, $packed_port, $cur);
    local ($remote_addr, @remote_ip, $remote_ip);
    local ($local_port, $remote_port);

    if ($port !~ /^\d+$/) {
        $port = (getservbyname($port, "tcp"))[2];
        $port = 443 unless ($port);
    }
    $proto = (getprotobyname('tcp'))[2];
    $remote_addr = (gethostbyname($remote_hostname))[4];
    if (!$remote_addr) {
        warn "Unknown host: $remote_hostname";
	return;
    }

    @remote_ip = unpack("C4", $remote_addr);
    $remote_ip = join(".", @remote_ip);
    print "Connecting to $remote_ip port $port.\n\n" if $opt_v;
    $socket_format = 'S n a4 x8';
    $local_port  = pack($socket_format, &AF_INET, 0, $local_host_ip);
    $remote_port = pack($socket_format, &AF_INET, $port, $remote_addr);
    socket($_[0], &AF_INET, &SOCK_STREAM, $proto) || die "socket: $!";
    bind($_[0], $local_port) || die "bind: $!";
    connect($_[0], $remote_port) || die "socket to $remote_ip: $!";
    $cur = select($_[0]);  

    $| = 1;                             # Disable buffering on socket.
    select($cur);

    $ctxserver = Net::SSLeay::CTX_new() or die_now("Failed to create SSL_CTX $!");
    Net::SSLeay::CTX_set_options($ctxserver, &Net::SSLeay::OP_ALL)
         and die_if_ssl_error("ssl ctx set options");
    ## Net::SSLeay::CTX_set_cipher_list($ctxserver, "RC4-MD5");
    $sslserver = Net::SSLeay::new($ctxserver) or die_now("Failed to create SSL $!");
    Net::SSLeay::set_fd($sslserver, fileno($_[0]));   # Must use fileno
    $res = Net::SSLeay::connect($sslserver) and die_if_ssl_error("ssl connect");
    print "Cipher `" . Net::SSLeay::get_cipher($sslserver) . "'\n" if $opt_v;
    print "Server certificate: ", Net::SSLeay::dump_peer_certificate($sslserver) if $opt_c;
}

sub readclient
{
	return <CLIENT> if $is_http;
	for (;;)
	{
		if ($_[0] && length($childbuf) >= $_[0])
		{
			$ret = $childbuf;
			$childbuf = "";
			return $ret;
		}
		if ($childbuf =~ s/([^\n]*\n)//sm)
		{
			print "CLIENT READ RETURNING FROM BUFFER\n" if $opt_v;
			return $1;
		}
		if (length $childbuf > 1024)
		{
			print "CLIENT READ RETURNING FROM BUFFER 1024\n" if $opt_v;
			$ret = $childbuf;
			$childbuf = "";
			return $ret;
		}
		print "STARTING CLIENT READ\n" if $opt_v;
		my $input = Net::SSLeay::read($sslclient, 1024);
		die_if_ssl_error("ssl client read");
		print "READ IN " . length($input) . "\n" if $opt_v;
		unless ($input)
		{
			$ret = $childbuf;
			$childbuf = "";
			return $ret;
		}
		$childbuf .= $input;
	}
}

sub readserver_undef_remain
{
	undef $readserver_remain;
}

sub readserver_set_remain
{
	$readserver_remain = @_[0] - length($serbuf);
	print "SET REMAIN TO $readserver_remain\n" if $opt_v;
}

sub readserver
{
	for (;;)
	{
		if ((!defined $readserver_remain || $url !~ /$ignore_url/) &&
			$serbuf =~ s/([^\n]*\n)//sm)
		{
			print "READ RET " . length($1) . "\n" if $opt_v;
			# chunked interpretation not needed since we add Connection: close
			# $readserver_remain = 0 if $next_to_last;
			# if ($is_chunked and ($1 eq "0\r\n"))
			# {
			# 	print "NEXT TO LAST CHUNK\n" if $opt_v;
			# 	 $next_to_last = 1;
			# }
			return $1;
		}
		if (length($serbuf) > 1024)
		{
			$ret = $serbuf;
			$serbuf = "";
			print "READX RET " . length($ret) . "\n" if $opt_v;
			return $ret;
		}
		if (defined $readserver_remain)
		{
			if ($readserver_remain > 0)
			{
				print "STARTING SERVER READ of $readserver_remain\n" if $opt_v;
				if ($is_http)
				{
					sysread(SERVER, $input, $readserver_remain ? $readserver_remain : 1024);
				}
				else
				{
					$input = Net::SSLeay::read($sslserver, $readserver_remain);
				}

				$readserver_remain -= length($input);
			}
			else
			{
				print "NO SERVER READ of $readserver_remain\n" if $opt_v;
				$input = "";
			}
		}
		else
		{
			print "STARTING SERVER READ\n" if $opt_v;
			if ($is_http)
			{
				$ret = sysread(SERVER, $input, 1024);
				print "SYSREAD RETURNED $ret, of 1024\n" if $opt_v;
			}
			else
			{
				$input = Net::SSLeay::read($sslserver, 1024);
			}
		}
		die_if_ssl_error("ssl server read");
		print "READ IN " . length($input) . "\n" if $opt_v;
		unless ($input)
		{
			$ret = $serbuf;
			$serbuf = "";
			print "READZ RET " . length($ret) . "\n" if $opt_v;
			return $ret;
		}
		$serbuf .= $input;
	}
}

sub writeserver
{
	print SERVER $_[0] if $is_http;
	if ($is_https)
	{
		print "STARTING SERVER WRITE " , length($_[0]), " bytes\n" if $opt_v;
		Net::SSLeay::write($sslserver, $_[0]);
		print "ENDING SERVER WRITE\n" if $opt_v;
		die_if_ssl_error("ssl server write");
	}
}

sub writeclient
{
	print CLIENT $_[0] if $is_http;
	if ($is_https)
	{
		print "STARTING CLIENT WRITE\n" if $opt_v;
		Net::SSLeay::write($sslclient, $_[0]);
		print "ENDING CLIENT WRITE\n" if $opt_v;
		die_if_ssl_error("ssl client write");
	}
}
